use anyhow::{bail, Result};
use log::info;
use reqwest::{
    multipart::{Form, Part},
    Body,
    Client,
};
use tokio::{fs::File, io::reader_stream};

use crate::{
    api::UpResp,
    util::{api, open_link},
};

pub async fn run(
    http: &Client,
    token: String,
    path: String,
    name: String,
    category: String,
    open: bool,
) -> Result<()> {
    if !api::cats(http).await?.contains(&category) {
        bail!(r#"category is invalid. use "cats" to list categories"#);
    }

    let res = http
        .post("https://data.tilera.xyz/api/jensmemes/upload")
        .multipart(
            Form::new()
                .text("category", category)
                .text("token", token)
                .part(
                    "file",
                    Part::stream(Body::wrap_stream(reader_stream({
                        info!("Opening file {}", &path);
                        File::open(path).await?
                    })))
                    .file_name(name),
                ),
        )
        .send()
        .await?;

    let status = res.status();
    let res = serde_json::from_slice::<UpResp>(&res.bytes().await?)?;

    println!("Server responded with code {}", status);

    if !open {
        println!();
    }

    for f in res.files {
        if open {
            open_link(&f).await?;
        } else {
            println!("{}", f);
        }
    }

    Ok(())
}
